/*****************************************************************************/
/*								             */
/* Author: Michael C. Nechyba					             */
/* Last modified: 9/20/98                   			             */
/*								             */
/* File: "bp.h" 						             */
/*								             */
/* This code is modified from the freely available NevProp v1.16 neural      */
/* network simulator (authored by Phil Goodman, David Rosen and Allen        */
/* Plummer).                                                                 */
/*								             */
/*****************************************************************************/

#define INT (0)
#define STR (1)
#define DBL (2)
#define FLT (3)
#define SHORT (4)

#define FLOAT  1
#define DOUBLE 2
#define STR_REG 256
#define STR_MAX 2048

#define LINEAR       0
#define SIGMOID      1  
#define ASYMSIGMOID  2  

#define WTS_EXTENSION "wts"
#define PRD_EXTENSION "prd"
#define ERR_EXTENSION "err"
#define RES_EXTENSION "log"
#define RANDOM_MAX 2147483647.

#define WTSEXIST "The specified .wts file does not exist; exiting.\n"
#define NOPATTERNS "NTraining & NTestingPatterns are both 0; exiting.\n"


#define printboth {if(SuppressConsole <= 0) printf(printargs); if (resfile) fprintf(resfile, printargs);}

typedef char *POINTER;

typedef struct parmentry {
  char *keyword;
  int  var_type;
  POINTER var_ptr;
} PARAMETERS;

/*********************** Prototype function declarations *********************/

int   main_loop(void);
void  INITIALIZE_GLOBALS(void);
void  GET_NETWORK_CONFIGURATION(void);
void  BUILD_DATA_STRUCTURES(int ninputs, int nhidden, int noutputs);
void  CONNECT_LAYERS(int start1, int end1, int start2, int end2);
void  GET_WEIGHTS(void);
void  MAKE_PREDICTIONS(void);
void  INIT_WEIGHTS(void);
float RANDOM_WEIGHT(float range);
void  PRINT_GLOBALS(FILE *stream, char *filename);
void  TRAIN(int times);
void  TRAIN_SUMMARY(void);
void  TEST_SUMMARY(void);
void  CHECK_IF_BEST(void);
void  TRAIN_ONE_EPOCH(void);
void  FORWARD_PASS(float *input);
float ACTIVATION(float sum, int unit_type);
float ERRFUN(float desired, float actual);
int   STOPPED_TRAINING(void);
void  CLEAR_SLOPES(void);
void  BACKWARD_PASS(float *goal);
float ACTIVATION_PRIME(float value, int unit_type);
void  UPDATE_WEIGHTS(void);
void  PRINT_RESULTS(char *label, int nPatterns, int errorBits, float SSE);
void  PRINT_BEST(void);
void  TEST_SAVE(void);
void  DUMP_WEIGHTS(void);
void  FREE_MEMORY(void);

int process_line(char *line, 
		 PARAMETERS *parameter_table, int n_parameters,
		 int alert_flag);
void get_parameters(char *file_name, PARAMETERS *parameter_table, 
		    int n_parameters);
int count_lines(char *file_name);
int file_exist(char *file_name);
void display_parameters(PARAMETERS *parameter_table, int n_parameters,
			FILE *fp);
void  protected_file(char *realfname, char *fname, 
		     char *extension, int protect_flag);
float rms(float sse, int n, int nout);
void  checkEOF(int scanfReturn);
